package framework.nrc

import framework.common._

/**
  * Shredding NRC extension
  */
trait ShredNRC extends NRC with BaseShredding with Label with Dictionary {

  /**
    * Shredded expression E is a pair of E^flat and E^dict
    */
  sealed trait SExpr {

    def flat: Expr 

    def dict: DictExpr

  }

  final case class ShredExpr(flat: Expr, dict: DictExpr) extends SExpr

  final case class ShredUdf(name:String, flat: Expr, dict: DictExpr, otp: Type, params: List[String], hint:String) extends SExpr {
    def tp:Type = dictTp(otp)
  }

  final case class ShredUnion(e1: BagExpr, e2: BagExpr) extends BagExpr {
     // Could be a heterogeneous union where labels in e1 and e2
     // for the same attribute encapsulate different parameters
     override def tp: BagType = e1.tp
  }

  final case class Lookup(lbl: LabelExpr, dict: BagDictExpr) extends BagExpr {
    def tp: BagType = dict.tp.flatTp
  }

  final case class ShredAssignment(name: String, rhs: SExpr)

  final case class ShredProgram(statements: List[ShredAssignment])

  object ShredProgram {
    def apply(statements: ShredAssignment*): ShredProgram = ShredProgram(List(statements: _*))
  }

}
